@@ -1,10 +1,3 @@
-/*
- * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can be
- * found in the LICENSE file.
- *
- * Windows platform-specific module methods for _psutil_windows
- */
 
 // Fixes clash between winsock2.h and windows.h
 #define WIN32_LEAN_AND_MEAN
@@ -31,7 +24,6 @@
 
 #include "common.h"
 #include "arch/windows/process_info.h"
-#include "arch/windows/process_handles.h"
 #ifndef __MINGW32__
 #include "arch/windows/ntextapi.h"
 #else
@@ -42,6 +34,7 @@
 #include <stdlib.h>
 
 #include "windows.h"
+
 /*
  * ============================================================================
  * Utilities
@@ -565,12 +558,14 @@ SEXP ps__proc_info(DWORD pid) {
   mem_private = 0;
 #endif
 
-  PROTECT(retlist = ps__build_named_list(
 #if defined(_WIN64)
-    "kkdddiKKKKKK" "kKKKKKKKKK",
+#define BUILD_NAMED_LIST_PROTO "kkdddiKKKKKK" "kKKKKKKKKK"
 #else
-    "kkdddiKKKKKK" "kIIIIIIIII",
+#define BUILD_NAMED_LIST_PROTO "kkdddiKKKKKK" "kIIIIIIIII"
 #endif
+
+  PROTECT(retlist = ps__build_named_list(
+    BUILD_NAMED_LIST_PROTO,
     "num_handles",         process->HandleCount,                  /* 0 */
     "ctx_switches",        ctx_switches,	                  /* 1 */
     "user_time",           user_time,		                  /* 2 */
@@ -729,7 +724,7 @@ SEXP psw__realpath(SEXP path) {
   return res;
 }
 
-SEXP ps__define_tcp_statuses() {
+SEXP ps__define_tcp_statuses(void) {
   SEXP result, names;
 
   PROTECT(result = allocVector(INTSXP, 13));
@@ -768,7 +763,7 @@ SEXP ps__define_tcp_statuses() {
 }
 
 SEXP ps__define_socket_address_families() {
-  SEXP afenv = PROTECT(Rf_allocSExp(ENVSXP));
+  SEXP afenv = PROTECT(ps_new_env());
 
   defineVar(install("AF_INET"), PROTECT(ScalarInteger(AF_INET)), afenv);
   UNPROTECT(1);
@@ -781,7 +776,7 @@ SEXP ps__define_socket_address_families() {
 }
 
 SEXP ps__define_socket_types() {
-  SEXP env = PROTECT(Rf_allocSExp(ENVSXP));
+  SEXP env = PROTECT(ps_new_env());
 
   defineVar(install("SOCK_STREAM"), PROTECT(ScalarInteger(SOCK_STREAM)), env);
   UNPROTECT(1);
@@ -793,6 +788,35 @@ SEXP ps__define_socket_types() {
   return env;
 }
 
+NTSTATUS (NTAPI *_NtQueryVirtualMemory) (
+  HANDLE ProcessHandle,
+  PVOID BaseAddress,
+  int MemoryInformationClass,
+  PVOID MemoryInformation,
+  SIZE_T MemoryInformationLength,
+  PSIZE_T ReturnLength
+);
+
+// A wrapper around LoadLibrary and GetProcAddress.
+PVOID ps__get_proc_address_from_lib(LPCSTR libname, LPCSTR procname) {
+  HMODULE mod;
+  FARPROC addr;
+
+  mod = LoadLibraryA(libname);
+  if (mod  == NULL) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  if ((addr = GetProcAddress(mod, procname)) == NULL) {
+    ps__set_error_from_windows_error(0);
+    FreeLibrary(mod);
+    ps__throw_error();
+  }
+
+  return addr;
+}
+
 SEXP ps__init(SEXP psenv, SEXP constenv) {
 
   /* Connection statuses */
@@ -805,5 +829,14 @@ SEXP ps__init(SEXP psenv, SEXP constenv) {
   /* Socket types */
   defineVar(install("socket_types"), ps__define_socket_types(), constenv);
 
+  /* errno values */
+  defineVar(install("errno"), ps__define_errno(), constenv);
+
+  /* load libs */
+  NtQueryVirtualMemory = ps__get_proc_address_from_lib(
+    "ntdll",
+    "NtQueryVirtualMemory"
+  );
+
   return R_NilValue;
 }
